function epm_matrix_experiments(idx, exps, single_exps, num_runs, model_idxs, do_exp, nTrain, num_features_wanted, num_params_wanted)
% Run experiment(s) for benchmark with index idx
% exps: matrix experiments for joint configuration/feature space (1: basic
% plots; 2: scaling experiments)
% single_exps: experiments for configuration space only (1: basic plots; 2:
% scaling experiments)
%
% Call this from the root directory.

% A new benchmark set can be added by setting the following below:
% tuningScenarios{end+1}: name of the scenario
% dist_files{end+1}: filename of a comma-separated Nx(M+1) runtime matrix, 
% where each of the N rows represents an instance, the first column holds 
% the instance name, and the remaining columns hold the runtimes of M
% configurations
% random_config_files{end+1}: filename holding the M configurations
% config_space_files{end+1}: description of the parameter configuration
% space, in the same format as for ParamILS
% feature_files{end+1}: filename containing the features for the
% benchmark's instances

% Define the models for the experiment.

% Ordering for scaling experiments:
% options_vec = {get_rf_default_options,get_regtree_default_options,get_gp_default_options,get_nn_options,get_lr_default_options,get_spore_options_rmse};

% Ordering for tables:
options_vec = {get_lr_default_options,get_spore_options_rmse,get_nn_options,get_gp_default_options,get_regtree_default_options,get_rf_default_options};
% options_vec = {get_lr_default_options,get_rf_default_options};
% options_vec = {get_rf_default_options};

% % C/V version, ordering for tables
% options_vec = {get_lr_cv_options,get_spore_cv_options_rmse,get_nn_cv_options,get_gp_default_options,get_regtree_default_options,get_rf_default_options,get_rf_cv_options}%,get_rf_default_options}
% 
% % C/V version, ordering for scaling experiments:
% % options_vec = {get_rf_cv_options,get_regtree_default_options,get_gp_default_options,get_nn_cv_options,get_spore_cv_options_rmse,get_lr_cv_options}%,get_rf_cv_options}
% 
% % options_vec = {get_rf_default_options}
% 
% % For censoring experiment
% options_vec = {get_rf_default_options_dropcensdata,get_rf_default_options_ignorecensinfo,get_rf_default_options_cens,get_rf_default_options_fill_in_samples};
% 
% options_vec = {get_rf_default_options};

options_vec = {get_lr_default_options,get_nn_options,get_gp_default_options,get_regtree_default_options,get_rf_default_options,get_rf_cv_options};

if 0% reverse order for plotting
    tmp = options_vec;
    options_vec = {};
    for i=length(tmp):-1:1
        options_vec{end+1} = tmp{i};
    end
end

do_plots = 1;
show_results_of_matrix_exps = 1;
capping_type = 'capslack'; 
capping_type = 'fixed'; 


mkdir('results');
if nargin < 6
    do_exp = 1;
    if nargin < 5
    model_idxs = 1:2;
        if nargin < 4
            num_runs = 1;
            if nargin < 3
                exps = [1,2,3,6];
                single_exps = [1,2,3,4];
            end
        end
    end
end
if isdeployed
    if nargin >= 6
        do_exp = str2double(do_exp);
        nTrain = str2double(nTrain);
        num_features_wanted = str2double(num_features_wanted);
        num_params_wanted = str2double(num_params_wanted); 
    end
    idx = str2double(idx);
    exps = str2double(exps);
    single_exps = str2double(single_exps);
    num_runs = str2double(num_runs);
    model_idxs = str2double(model_idxs);
    do_plots = 0;
end

rootdir = '';
tuningScenarios = {};
dist_files = {};
random_config_files = {};
config_space_files = {};
feature_files = {};

tuningScenarios{end+1} = 'CPLEX12-cat-BIGMIX';
dist_files{end+1} = strcat(rootdir, 'data/MIP/1000samples-CPLEX-BIGMIX-all1510inst-results.txt');
random_config_files{end+1} = strcat(rootdir, 'data/MIP/1000samples-algocplex12-milp-runobjruntime-overallobjmean10-runs1000-time300.0-length2147483647_0.txt');
config_space_files{end+1} = strcat(rootdir, 'data/MIP/cplex12-params-CPAIOR-space.txt');
feature_files{end+1} = strcat(rootdir, 'data/MIP/BIGMIX-train_test-features-withfilename.csv');

tuningScenarios{end+1} = 'CPLEX12-cat-CORLAT';
dist_files{end+1} = strcat(rootdir, 'data/MIP/1000samples-CPLEX-CORLAT-train_test_inst-results.txt');
random_config_files{end+1} = strcat(rootdir, 'data/MIP/1000samples-algocplex12-milp-runobjruntime-overallobjmean10-runs1000-time300.0-length2147483647_0.txt');
config_space_files{end+1} = strcat(rootdir, 'data/MIP/cplex12-params-CPAIOR-space.txt');
feature_files{end+1} = strcat(rootdir, 'data/MIP/CORLAT-train_test-features-withfilename.csv');

tuningScenarios{end+1} = 'CPLEX12-cat-REG';
dist_files{end+1} = strcat(rootdir, 'data/MIP/1000samples-CPLEX-CATS_REG-1000train-1000test-results.txt');
random_config_files{end+1} = strcat(rootdir, 'data/MIP/1000samples-algocplex12-milp-runobjruntime-overallobjmean10-runs1000-time300.0-length2147483647_0.txt');
config_space_files{end+1} = strcat(rootdir, 'data/MIP/cplex12-params-CPAIOR-space.txt');
feature_files{end+1} = strcat(rootdir, 'data/MIP/REG-train_test-features-withfilename.csv');

tuningScenarios{end+1} = 'CPLEX12-cat-RCW';
dist_files{end+1} = strcat(rootdir, 'data/MIP/1000samples-CPLEX-RCW-990train-990test-results.txt');
random_config_files{end+1} = strcat(rootdir, 'data/MIP/1000samples-algocplex12-milp-runobjruntime-overallobjmean10-runs1000-time300.0-length2147483647_0.txt');
config_space_files{end+1} = strcat(rootdir, 'data/MIP/cplex12-params-CPAIOR-space.txt');
feature_files{end+1} = strcat(rootdir, 'data/MIP/RCW-train_test-features-withfilename.csv');

tuningScenarios{end+1} = 'CPLEX12-cat-CORLAT-REG';
dist_files{end+1} = strcat(rootdir, 'data/MIP/1000samples-CPLEX-CORLAT-REG-results.txt');
random_config_files{end+1} = strcat(rootdir, 'data/MIP/1000samples-algocplex12-milp-runobjruntime-overallobjmean10-runs1000-time300.0-length2147483647_0.txt');
config_space_files{end+1} = strcat(rootdir, 'data/MIP/cplex12-params-CPAIOR-space.txt');
feature_files{end+1} = strcat(rootdir, 'data/MIP/CORLAT-REG-features.csv');

tuningScenarios{end+1} = 'CPLEX12-cat-CORLAT-REG-RCW';
dist_files{end+1} = strcat(rootdir, 'data/MIP/1000samples-CPLEX-CORLAT-REG-RCW-results.txt');
random_config_files{end+1} = strcat(rootdir, 'data/MIP/1000samples-algocplex12-milp-runobjruntime-overallobjmean10-runs1000-time300.0-length2147483647_0.txt');
config_space_files{end+1} = strcat(rootdir, 'data/MIP/cplex12-params-CPAIOR-space.txt');
feature_files{end+1} = strcat(rootdir, 'data/MIP/CORLAT-REG-RCW-features.csv');

tuningScenarios{end+1} = 'SPEAR-ibm-al';
dist_files{end+1} = strcat(rootdir, 'data/SAT/1000samples-SPEAR-IBM-all765inst-results.txt');
random_config_files{end+1} = strcat(rootdir, 'data/SAT/1000samples-algospear1.2.1.1-runobjruntime-overallobjmean10-runs1000-time300.0-length2147483647_0.txt');
config_space_files{end+1} = strcat(rootdir, 'data/SAT/spear-params.txt');
feature_files{end+1} = strcat(rootdir, 'data/SAT/IBM-ALL-feat.csv');

tuningScenarios{end+1} = 'SPEAR-swv-al';
dist_files{end+1} = strcat(rootdir, 'data/SAT/1000samples-SPEAR-SWV-all604inst-results.txt');
random_config_files{end+1} = strcat(rootdir, 'data/SAT/1000samples-algospear1.2.1.1-runobjruntime-overallobjmean10-runs1000-time300.0-length2147483647_0.txt');
config_space_files{end+1} = strcat(rootdir, 'data/SAT/spear-params.txt');
feature_files{end+1} = strcat(rootdir, 'data/SAT/SWV-feat.csv');

tuningScenarios{end+1} = 'SPEAR-ibm-swv-al';
dist_files{end+1} = strcat(rootdir, 'data/SAT/1000samples-SPEAR-IBM-SWV-results.txt');
random_config_files{end+1} = strcat(rootdir, 'data/SAT/1000samples-algospear1.2.1.1-runobjruntime-overallobjmean10-runs1000-time300.0-length2147483647_0.txt');
config_space_files{end+1} = strcat(rootdir, 'data/SAT/spear-params.txt');
feature_files{end+1} = strcat(rootdir, 'data/SAT/IBM-SWV-feat.csv');

tuningScenarios{end+1} = 'LKH-TSPLIB';
%dist_files{end+1} = strcat(rootdir, 'data/TSP/1000-randomsamples-LKH-runobjruntime-overallobjmean10-time300-algowrapper5.pl-traj-matrix-results.txt');
dist_files{end+1} = strcat(rootdir, 'data/TSP/1000-randomsamples-LKH-runobjruntime-overallobjmean10-time300-algoLKHWrapper.pl-traj-matrix-both-ok-results.csv');
random_config_files{end+1} = strcat(rootdir, 'data/TSP/1000-randomsamples-LKH-runobjruntime-overallobjmean10-time300-algoLKHWrapper.pl-traj.txt');
config_space_files{end+1} = strcat(rootdir, 'data/TSP/LKH-params.txt');
% feature_files{end+1} = strcat(rootdir, 'data/TSP/TSPLIB_feat_all.csv');
feature_files{end+1} = strcat(rootdir, 'data/TSP/TSPLIB_feat_both_ok.csv');

tuningScenarios{end+1} = 'LKH-TSPLIB';
dist_files{end+1} = strcat(rootdir, 'data/TSP/1000-randomsamples-LKH-runobjruntime-overallobjmean10-time300-algowrapper5.pl-traj-matrix-ours-ok-results.csv');
random_config_files{end+1} = strcat(rootdir, 'data/TSP/1000-randomsamples-LKH-runobjruntime-overallobjmean10-time300-algowrapper5.pl-traj.txt');
config_space_files{end+1} = strcat(rootdir, 'data/TSP/LKH-params.txt');
feature_files{end+1} = strcat(rootdir, 'data/TSP/TSPLIB_feat_ours_ok.csv');

if do_exp
    for i=idx
        for exp=exps
            model_perf_scenario_clean(options_vec, dist_files{i}, random_config_files{i}, feature_files{i}, tuningScenarios{i}, config_space_files{i}, 300, exp, -1, capping_type, num_runs, model_idxs, nTrain, num_features_wanted, num_params_wanted);
        end
        for singleExp=single_exps
            model_perf_scenario_clean(options_vec, dist_files{i}, random_config_files{i}, feature_files{i}, tuningScenarios{i}, config_space_files{i}, 300, 0, singleExp, capping_type, num_runs, model_idxs, nTrain, num_features_wanted, num_params_wanted);
        end
        close all;
    end
end

if do_plots
    if length(exps) == 1 && exps(1) == 7
        for i = idx
            plot_model_perf_capslack(tuningScenarios{i}, capping_type)
        end
        return
    end
    is_first_idx = 1;
    for num_tuningScenario=idx
        tuningScenario = tuningScenarios{num_tuningScenario};
        outdomain = strcat('results/matrix/', tuningScenarios{num_tuningScenario}, '/');
        
        for singleExp=single_exps
            plot_ehm_results(options_vec, strcat('params-',tuningScenarios{num_tuningScenario}), singleExp, num_tuningScenario);
        end
        
        for exp = exps
            if exp == 1
                tuningScenParams = strcat('params-', tuningScenario);
                timesToLearn = [];
                for model_idx = model_idxs
                    options = options_vec{model_idx};
                    
                    %% Detailed matrix experiments.
                    nTrain = 10000;
                    tuningScen_with_numtrain = strcat(tuningScenario,'-',num2str(nTrain));
                    if show_results_of_matrix_exps
                        filename = ['results/matrix/2d/', tuningScen_with_numtrain, '-', options.unique_model_name, '-preds'];
                        load(filename);
                        tuningScenario = tuningScenarios{num_tuningScenario}; % to undo load.
                         
                        fprintf('Overall performance in matrix prediction: RMSE = %f, LL=%f, CC=%f, CC_rank=%f\n', [rmse, ll, cc, cc_rank]);
                        % 4 sub-experiments
                        learnTime = timeToLearn;
                        % subplot(2,2,1);
                        results{end+1} = predict_submatrix(matrix, fullpredmatrix, fullpredmatrixvar, ind1(find(ind1 <= nInstTrain)), ind2(find(ind2 <= nThetaTrain)), tuningScenario, '-trainC_trainI', options.unique_model_name, learnTime, plotMatrices);
                        close all;
                        
                        %subplot(2,2,2);
                        results{end+1} = predict_submatrix(matrix, fullpredmatrix, fullpredmatrixvar, ind1(find(ind1 <= nInstTrain)), ind2(find(ind2 > nThetaTrain)), tuningScenario, '-testC_trainI', options.unique_model_name, learnTime, plotMatrices);
                        close all;
                        
                        %subplot(2,2,3);
                        results{end+1} = predict_submatrix(matrix, fullpredmatrix, fullpredmatrixvar, ind1(find(ind1 >  nInstTrain)), ind2(find(ind2 <= nThetaTrain)), tuningScenario, '-trainC_testI', options.unique_model_name, learnTime, plotMatrices);
                        close all;
                        
                        %subplot(2,2,4);
                        results{end+1} = predict_submatrix(matrix, fullpredmatrix, fullpredmatrixvar, ind1(find(ind1 > nInstTrain)), ind2(find(ind2 > nThetaTrain)), tuningScenario, '-testC_testI', options.unique_model_name, learnTime, plotMatrices);
                        close all;
                    end
                end
                
                %% Load statistics of matrix experiments.
                for model_idx = model_idxs
                    options = options_vec{model_idx};
                    filename = strcat(outdomain, options.unique_model_name, '_matrix_preds.mat');
                    %             fprintf(strcat(['Loading results for model ', options.unique_model_name, ' from file ', filename, '\n']));
                    load(filename, 'results', 'timeToLearn', 'timeToPredict', 'nTrain');
                    all_results{model_idx} = results;
                    all_timeToLearn{model_idx} = timeToLearn;
                    all_timeToPredict{model_idx} = timeToPredict;
                end
                
                stats_format = 2;
                switch stats_format
                    case 1 
                        fprintf('\\hline\n');
                        fprintf(strcat([' & & & CC']));
                        for quadr=1:4
                            fprintf(' & ');
                            values = [];
                            for model_idx = 1:length(options_vec)
                                values(end+1) = all_results{model_idx}{quadr}(3);
                            end
                            n_output(values, 0);
                        end
                        fprintf('\\\\\n');

                        fprintf(strcat([tuningScenario, ' & ']));
                        n_output(all_timeToLearn, 1);

                        fprintf(strcat(['& RMSE']));
                        for quadr=1:4
                            fprintf(' & ');
                            values = [];
                            for model_idx = model_idxs
                                values(end+1) = all_results{model_idx}{quadr}(1);
                            end
                            n_output(values, 1);
                        end
                        fprintf('\\\\\n');

                        fprintf(strcat([' & & & LL']));
                        for quadr=1:4
                            fprintf(' & ');
                            values = [];
                            for model_idx = model_idxs
                                values(end+1) = all_results{model_idx}{quadr}(2);
                            end
                            n_output(values, 0);
                        end
                        fprintf('\\\\\n');
                
                    case 2
                        rmses = [];
                        lls = [];
                        ccs = [];
                        learnTime = [];

                        modelNames = get_model_names(options_vec,0);
                        if is_first_idx
                            header_output(modelNames, {'RMSE', 'Time to learn [s]'});
%                             header_output(modelNames, {'Pearson correlation coefficient', 'Log-likelihood'});
%                             header_output(modelNames, {'RMSE', 'Pearson correlation coefficient'});
                            is_first_idx = 0;
                        end
                           
                        quadr = 4;
                        model_idx_to_use = model_idxs
                        for j=1:length(model_idx_to_use)
                            model_idx = model_idx_to_use(j);
                            rmses(j) = all_results{model_idx}{quadr}(1);
                            lls(j) = all_results{model_idx}{quadr}(2);
                            ccs(j) = all_results{model_idx}{quadr}(3);
                            learnTime(j) = all_timeToLearn{model_idx};
                        end

                        fprintf(strcat(fix_name(tuningScenario), ' & '));
                        n_output(rmses,1);
                        fprintf(' & ');
                        n_output(learnTime,1);

%                         n_output(ccs,0);
%                         fprintf(' & ');
% %                         lls = lls([4,6,7]); % only defined for PP and RF
%                         n_output(lls,0);

%                         n_output(rmses,1);
%                         fprintf(' & ');
%                         n_output(learnTime,1);

                        
                        fprintf('\\\\\n');
                        fprintf('\\hline\n');
                        
                        
                
                    case 3
    %                     fprintf('======= All 4 combinations of training/test insts and configs =======\n\n');
                        modelNames = get_model_names(options_vec,0);
                        if is_first_idx
                            header_output(modelNames, {'Training configurations', 'Test configurations'}, ' Instances & ', ' & ');
                            is_first_idx = 0;
                        end

                        fprintf(strcat([fix_name(tuningScenario), ' & Training & ']));
                        for quadr=1:2
                            outresults = [];
                            for model_idx=1:length(options_vec)
                                outresults(model_idx) = all_results{model_idx}{quadr}(1);%rmse
                            end
                            n_output(outresults, 1);
                            if quadr == 1
                                fprintf(' & ');
                            end
                        end
                        fprintf('\\\\\n');

                        fprintf(strcat(['~ & Test & '])); 
                        for quadr=3:4
                            outresults = [];
                            for model_idx=1:length(options_vec)
                                outresults(model_idx) = all_results{model_idx}{quadr}(1);
                            end
                            n_output(outresults, 1);
                            if quadr == 3
                                fprintf(' & ');
                            end
                        end
                        fprintf('\\\\\n');
                        fprintf('\\hline\n');
                end
            end
        end
        
        if exp==2
            outdomain = strcat('results/matrix/', tuningScenarios{num_tuningScenario}, '/');

            numTrains = [10,10*sqrt(10),100,100*sqrt(10),1000,1000*sqrt(10),10000,10000*sqrt(10),100000];
        %     if isTinyTest
        %         numTrains = [10,10*sqrt(10)];
        %     end

            for model_idx = 1:length(options_vec)
                options = options_vec{model_idx};

                model_names{model_idx} = options.unique_model_name;
                for i=1:length(numTrains)
                    numTrain = numTrains(i);
                    numTrain = floor(numTrain);
                    filename = strcat(outdomain, options_vec{model_idx}.unique_model_name, '-nTrain', num2str(numTrain), '_matrix-pred-scale-result.mat');
                    try
                        load(filename, 'results', 'timeToLearn', 'timeToPredict', 'numTrains');            
                        ccs = [];
                        for j=1:10
                            ccs(end+1) = results{i,j}{4}(3);
                        end
                        cc_avg(i,model_idx) = mean(ccs);
                        cc_std(i,model_idx) = std(ccs);
                    catch ME
                        cc_avg(i,model_idx) = inf;
                        cc_std(i,model_idx) = inf;
                    end
                end            
            end
            numTrains = ceil(numTrains);
    %         numTrains = numTrains(3:end);
    %         cc_avg = cc_avg(3:end,:);
    %         cc_std = cc_std(3:end,:);
    %         modelNames = get_model_names(options_vec);
            modelNames = model_names;
    %        plot_Scaling_N(numTrains, cc_avg, cc_std, options_vec, 'Correlation Coefficient', modelNames, tuningScenarios{num_tuningScenario}, 'NorthWest', 0)
             plot_Scaling_N(numTrains, cc_avg, cc_std, options_vec, 'Correlation Coefficient', modelNames, tuningScenarios{num_tuningScenario}, 'SouthEast', 0)

        end
    end
end

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%% SCALING OF MATRIX PREDICTIONS WITH NUMBER OF TRAINING DATA POINTS
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     if scaling_matrix
%         for i=idx
%             outdomain = strcat('results/matrix/', tuningScenarios{i});
%             savefile = strcat(outdomain, '-matrix-pred-scale');
%             load(savefile);
%             numTrains = floor(numTrains);
%
%     %        model_idx_to_use = 1:length(options_vec);
%             model_idx_to_use = [2,3,4,5];
%             model_names = get_model_names(options_vec(model_idx_to_use));
%             cc_avg = [];
%             cc_std = [];
%             for m=1:length(model_idx_to_use)
%                 for j=1:length(numTrains)
%                     ccs = [];
%                     for k=1:size(results,3)
%                         res = results{model_idx_to_use(m),j,k};
%                         if iscell(res)
%                             ccs(end+1) = res{4}(3);
%                         else
%                             ccs(end+1) = -inf;
%                         end
%                     end
%                     cc_avg(j,m) = mean(ccs);
%                     cc_std(j,m) = std(ccs);
%                 end
%             end
%             numTrains = numTrains(3:end);
%             cc_avg = cc_avg(3:end,:);
%             cc_std = cc_std(3:end,:);
%             plot_Scaling_N(numTrains, cc_avg, cc_std, options_vec(model_idx_to_use), 'Correlation Coefficient', model_names, tuningScenarios{i}, 'SouthEast', 0)
% %         rmse_avg = [];
% %         rmse_std = [];
% %         for m=1:length(model_idx_to_use)
% %             for j=1:length(numTrains)
% %                 rmses = [];
% %                 for k=1:size(results,3)
% %                     rmses(end+1) = results{model_idx_to_use(m),j,k}{4}(1);
% %                 end
% %                 rmse_avg(j,m) = mean(rmses);
% %                 rmse_std(j,m) = std(rmses);
% %             end
% %         end
% %         plot_Scaling_N(numTrains, rmse_avg, rmse_std, options_vec(model_idx_to_use), 'RMSE', model_names, tuningScenarios{i}, 'SouthEast', 0)
%         end
%     end
% end

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%% VARIABLE IMPORTANCE.
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for exp = exps(:)'
    if exp==4
        for i=idx
            savefile = strcat('results/vimp/vimp_configspace_', tuningScenarios{i});
            output_vimps(savefile, options_vec(model_idxs), strcat('results/fwd_sel/configspace_',tuningScenarios{i}), 'Parameter subset size');
        end
    end

    if exp==5
        for i=idx
            savefile = strcat('results/vimp/vimp_featurespace_', tuningScenarios{i});
            output_vimps(savefile, options_vec(model_idxs), strcat('results/fwd_sel/vimp_featurespace_',tuningScenarios{i}), 'Feature subset size');
        end
    end

    if exp==6 || exp == 8
        for i=idx
            nTrain = 10000;
            nValid = nTrain/2;
            nTest = 5000;
            savefile = strcat('results/vimp/vimp_matrix_', num2str(nTrain),'_', num2str(nValid),'_',num2str(nTest),'_', tuningScenarios{i});
            prefix = strcat('results/fwd_sel/vimp_matrix_', num2str(nTrain),'_', tuningScenarios{i});
            if exp == 8
                savefile = strcat('results/vimp/vimp_matrix_np_', num2str(num_features_wanted), '_nf', num2str(num_params_wanted), '_', num2str(nTrain),'_', num2str(nValid),'_',num2str(nTest),'_', tuningScenarios{i});
                prefix = strcat('results/fwd_sel/vimp_matrix_np_', num2str(num_features_wanted), '_nf', num2str(num_params_wanted), '_', num2str(nTrain),'_', tuningScenarios{i});
            end
            fprintf([fix_name(tuningScenarios{i}) ' & ']);
            output_vimps(savefile, options_vec(model_idxs), prefix, 'Feature/parameter subset size');
        end
    end

 
    if exp == 16 || exp == 18
        for num_tuningScenario=idx
            tuningScenario = tuningScenarios{num_tuningScenario};
            fprintf([fix_name(tuningScenario) ' & ']);

            % This requires that experiment 6 (for 16) or 8 (for 18) has run before.
            nValid = nTrain/2;
            nTest = 5000;
            savefile = strcat('results/vimp/vimp_matrix_', num2str(nTrain),'_', num2str(nValid),'_',num2str(nTest),'_', tuningScenario);
            output_constraint_string = '';
            if exp == 18
                savefile = strcat('results/vimp/vimp_matrix_np_', num2str(num_features_wanted), '_nf', num2str(num_params_wanted), '_', num2str(nTrain),'_', num2str(nValid),'_',num2str(nTest),'_', tuningScenario);
                output_constraint_string = ['_nf', num2str(num_features_wanted), '_np', num2str(num_params_wanted)];
            end
            for model_idx = model_idxs(:)'
                options = options_vec{model_idx};
                for num_run = num_runs(:)'
                    this_savefile = strcat([savefile, '_', num2str(num_run)]);
                    savefilename = strcat([this_savefile, '_', options_vec{model_idx}.unique_model_name]);
                    try
                        load(savefilename, 'vimp');
                        all_vimps{model_idx,num_run} = vimp; %#ok<AGROW>
                    catch ME
                        all_vimps{model_idx,num_run} = -inf; %#ok<AGROW>
                    end
                end    
                vimp = all_vimps{model_idx,num_run}; % only plot the last one.

                for subsetsize = 5; %[1:10,1000000]
%                     filename = strcat('results/matrix/', tuningScenario, '/', options_vec{model_idx}.unique_model_name, '_matrix_preds_sub_', num2str(subsetsize), '_nTrain', num2str(nTrain), '.mat');
                    filename = strcat('results/matrix/', tuningScenario, '/', options_vec{model_idx}.unique_model_name, '_matrix_preds_sub_', num2str(subsetsize), '_nTrain', num2str(nTrain), '.mat');
                    if exp == 18
                        filename = strcat('results/matrix/', tuningScenario, '/', options_vec{model_idx}.unique_model_name, '_matrix_np_', num2str(num_features_wanted), '_nf', num2str(num_params_wanted), '_preds_sub_', num2str(subsetsize), '_nTrain', num2str(nTrain), '.mat');
                        if subsetsize < 1000000 && subsetsize > num_features_wanted + num_params_wanted
                            continue
                        end
                    end            
                    load(filename, 'results', 'timeToLearn', 'timeToPredict', 'nTrain', 'model', 'matrix', 'trainC', 'validC', 'testC', 'trainI', 'validI', 'testI', 'subFunc', 'subTheta', 'sub_all_features', 'subNames');

                    close all;
                    if size(subTheta,2) > 0
                        %% Study performances of best partial instantiations.
                        % Find test configurations with these partial settings and compare
                        % their average performance (marginal across instances) vs. other
                        % test configurations.

                        % 1: Get all unique instantiations of the subset's parameters
                        % (Simple solution: with high probability subTheta covers all of them if numParams is small)
                        testUniqueSubTheta = unique(subTheta(testC,:), 'rows');

                        % 2: Predict marginal performance for them on validation data.
                        valPreds = [];
                        for i = 1:length(validI)
                            X = [testUniqueSubTheta, repmat(sub_all_features(validI(i),:), [size(testUniqueSubTheta,1),1])];
                            valPreds(i,:) = 10.^applyModel(model, X, 0);
                        end
                        meanValPreds = mean(valPreds,1);

                        % 3: Identify minimum of predicted validation performance, 
                        % and identify test parameter settings matching this partial instantiation
                        [tmp, min_idx] = min(meanValPreds);
                        
                        % output best setting of the parameters 
                        best_param_values = testUniqueSubTheta(min_idx,:);
                        for i=1:length(subFunc.all_values)
                            subNames{i};
                            subFunc.all_values{i}(best_param_values(i));
                        end
                        
                        bestValTheta = testUniqueSubTheta(min_idx,:);
                        otherThetas = setdiff(testUniqueSubTheta, bestValTheta, 'rows');

                        idxs_of_matching_test_thetas = find(ismember(subTheta(testC,:), bestValTheta, 'rows'));
    %                     idxs_of_matching_test_thetas = intersect(idxs_of_matching_thetas, testC);
                        idxs_of_other_test_thetas = setdiff(testC, idxs_of_matching_test_thetas);

                        mean_test_performance_of_best_val_theta = mean(matrix(idxs_of_matching_test_thetas, testI), 1);
    %                     mean_test_performance_of_other_theta = mean(matrix(idxs_of_other_test_thetas, testI), 1);
                        mean_test_performance_of_any_theta = mean(matrix(testC, testI), 1);

                        % 4: Plot marginal performance across test instances for those and
                        % for the remaining test parameters in a box plot each.
                        rep_tuningScenario = regexprep(tuningScenario, '-cat', '');
                        rep_tuningScenario = regexprep(rep_tuningScenario, '12', '');
                        rep_tuningScenario = regexprep(rep_tuningScenario, '-al', '');

                        test_performances_of_best_val_theta = mean(matrix(idxs_of_matching_test_thetas, testI), 2);
% %         %                 test_performances_of_other_thetas = mean(matrix(idxs_of_other_test_thetas, testI), 2);
                        test_performances_of_all_thetas = mean(matrix(testC, testI), 2);

%                         figure
%                         plotdata = [test_performances_of_all_thetas(:); test_performances_of_best_val_theta(:)];
%                         plotlabels = cell(length(plotdata),1);
%                         for i=1:length(test_performances_of_all_thetas)
%                             plotlabels{i} = 'all';
%                         end
%                         for i=1:length(test_performances_of_best_val_theta)
%                             plotlabels{i+length(test_performances_of_all_thetas)} = 'best';
%                         end
%                         hb = boxplot(plotdata, plotlabels);
% % %         %                 hyLabel = ylabel('Runtime for instances [CPU s]')
% % %         %                 set(hyLabel, 'FontSize', 12); % , 'VerticalAlignment', 'bottom'
% 
%                         hxLabel = xlabel(rep_tuningScenario);
%                         set(hxLabel, 'FontSize', 11); % , 'VerticalAlignment', 'bottom'
%                         set(gcf, 'Outerposition', [1000,500,150,300]);
%                         save_gca_as_fig_and_pdf(strcat('results/speedups/avg_perf_', tuningScenario, '_', options_vec{model_idx}.unique_model_name, output_constraint_string, '-sub', num2str(subsetsize), '-ntrain', num2str(nTrain)));
 
                        % 5: Predict performance for them on test data,
                        % and select best setting for each instance.
                        testPreds = [];
                        bestTestTheta = [];
                        for i = 1:length(testI)
                            X = [testUniqueSubTheta, repmat(sub_all_features(testI(i),:), [size(testUniqueSubTheta,1),1])];
                            testPreds(i,:) = applyModel(model, X, 0);
                            [tmp, min_idx] = min(testPreds(i,:));
                            bestTestTheta(i,:) = testUniqueSubTheta(min_idx,:);
                        end
                        test_performances_of_best_per_instance_choices = cell(length(testI));
                        mean_test_performance_of_best_per_instance_choices = -ones(length(testI),1);
                        for i=1:length(testI)
                            this_idxs_of_matching_test_thetas = find(ismember(subTheta, bestTestTheta(i,:), 'rows'));
                            this_idxs_of_matching_test_thetas = intersect(this_idxs_of_matching_test_thetas, testC);
                            test_performances_of_best_per_instance_choices{i} = matrix(this_idxs_of_matching_test_thetas, testI(i));
                            mean_test_performance_of_best_per_instance_choices(i) = mean(test_performances_of_best_per_instance_choices{i});
                        end

                        
%                         solved_idx =
%                         intersect(intersect(find(mean_test_performance_of_any_theta < 300), find(mean_test_performance_of_best_val_theta < 300)), find(mean_test_performance_of_best_per_instance_choices<300));

                        solved_idx = find(mean_test_performance_of_any_theta < 300);
                        mean_of_any = mean(mean_test_performance_of_any_theta(solved_idx))
                        mean_of_best_fixed = mean(mean_test_performance_of_best_val_theta(solved_idx))
                        mean_of_best_p_i = mean(mean_test_performance_of_best_per_instance_choices(solved_idx))
%                         
%                         % Boxplot showing mean performance for each instance.
%                         figure
%                         plotdata = [mean_test_performance_of_any_theta(:); mean_test_performance_of_best_val_theta(:); mean_test_performance_of_best_per_instance_choices(:)]; %mean_test_performance_of_other_theta(:); 
%                         plotlabels = cell(length(plotdata),1);
%                         for i=1:length(testI)
%                             plotlabels{i} = 'all';
%                         end
%                         for i=1:length(testI)
%                             plotlabels{i+length(testI)} = 'best';
%                         end
%                         for i=1:length(testI)
%                             plotlabels{i+2*length(testI)} = 'p.i.';
%                         end
%         %                 for i=1:length(testI)
%         %                     plotlabels{i+3*length(testI)} = 'other';
%         %                 end
%                         boxplot(plotdata, plotlabels);
%                         hxLabel = xlabel(rep_tuningScenario);
%                         set(hxLabel, 'FontSize', 11); % , 'VerticalAlignment', 'bottom'
%                         set(gcf, 'Outerposition', [1000,500,150,300]);
%                         save_gca_as_fig_and_pdf(strcat('results/speedups/inst_perf_', tuningScenario, '_', options_vec{model_idx}.unique_model_name, output_constraint_string, '-sub', num2str(subsetsize), '-ntrain', num2str(nTrain)));
% 
                        % Boxplot showing speedups over mean.
                        figure
                        speedup_best_single = mean_test_performance_of_any_theta(:)./mean_test_performance_of_best_val_theta(:);
                        speedup_best_perinst = mean_test_performance_of_any_theta(:)./mean_test_performance_of_best_per_instance_choices(:);
                        plotdata = log10([speedup_best_single; speedup_best_perinst]);
                        plotlabels = cell(length(plotdata),1);
                        for i=1:length(testI)
                            plotlabels{i} = 'best';
                        end
                        for i=1:length(testI)
                            plotlabels{i+length(testI)} = 'p.i.';
                        end
                        boxplot(plotdata, plotlabels);
                        hLine = line([0,1000],[0,0]);
                        set(hLine, 'LineStyle', '--', 'Color', 'g');
                        axis([0.5,2.5,-2,4]);
                        
%                         hxLabel = xlabel(rep_tuningScenario);
%                         set(hxLabel, 'FontSize', 11); % , 'VerticalAlignment', 'bottom'
                        set(gcf, 'Outerposition', [1000,500,150,300]);
                        mkdir('results/speedups');
                        save_gca_as_fig_and_pdf(strcat('results/speedups/speedups_', tuningScenario, '_', options_vec{model_idx}.unique_model_name, output_constraint_string, '-sub', num2str(subsetsize), '-ntrain', num2str(nTrain)));

%                         figure
%                         loglog(mean_test_performance_of_any_theta, mean_test_performance_of_best_val_theta, 'k.');
%                         hold on
%                         mini = min(min(mean_test_performance_of_any_theta), min(mean_test_performance_of_best_val_theta)) / 1.3;
%                         maxi = max(max(mean_test_performance_of_any_theta), max(mean_test_performance_of_best_val_theta)) * 1.3;
%                         hLine = line([mini,maxi],[mini,maxi]);
%                         hBest = loglog(mean_test_performance_of_any_theta, mean_test_performance_of_best_val_theta, 'k.', 'MarkerSize', 10);
%                         hPi = loglog(mean_test_performance_of_any_theta, mean_test_performance_of_best_per_instance_choices, 'rx', 'MarkerSize', 8);
%                         hxLabel = xLabel('Avg. perf. [CPU s]');
%                         hyLabel = yLabel('Perf. of best and best per instance [CPU s]');
%                         set(hxLabel, 'FontSize', 14);
%                         set(hyLabel, 'FontSize', 14);
%                         axis([mini,maxi,mini,maxi]);
%                         legend([hBest,hPi], {'Single best', 'Per-instance'}, 'Location', 'NorthWest');
%                         save_gca_as_fig_and_pdf(strcat('results/speedups/scatter_', tuningScenario, '_', options_vec{model_idx}.unique_model_name, output_constraint_string, '-sub', num2str(subsetsize), '-ntrain', num2str(nTrain)));
                    end
                end
            end
        end
    end
end

% tuningScenarios{end+1} = 'SAPS-c-SWGCP-PAR1';
% dist_files{end+1} = strcat(rootdir, 'data/SAT/random-SWGCPsat_SAPScont-5s-1000train-1000test.txt');
% random_config_files{end+1} = strcat(rootdir, 'data/SAT/1000samples-algosaps-runobjruntime-overallobjmean10-time300.0-length2147483647_0.txt');
% algos{end+1} = 'saps';
% feature_files{end+1} = strcat(rootdir, 'data/SAT/SWGCP-SAT-feat.csv');


% % %NEW:
% tuningScenario = 'SAPS-SWGCP-cont';
% if isunix
%     dist_file = '/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/saps-sw/random-SW-JAIR-results-1000train-1000test.txt';
%     random_config_file = '/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/saps-newqcp/1000random_samples-runobjruntime-overallobjmean10-time5.0-algosaps-later-clean.txt';
% else
%     dist_file = 'Z:\arrowspace\ParamILS\saps-sw\random-SW-JAIR-results-1000train-1000test.txt';
%     random_config_file = 'Z:\arrowspace\ParamILS\saps-newqcp\1000random_samples-runobjruntime-overallobjmean10-time5.0-algosaps-later-clean.txt';
% end
% %analyze_scenario(tuningScenario, dist_file, 5);
% model_perf_scenario(dist_file, random_config_file, tuningScenario, 5);
% %cv_marginal_pred_exploratory('/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/saps-sw/random-SW-JAIR-results-1000train-1000test.txt', '/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/saps-newqcp/1000random_samples-runobjruntime-overallobjmean10-time5.0-algosaps-later-clean.txt', 'SAPS-SWGCP-cont', 5);
% clear all;
% close all hidden;
%
%
% if 0
%     %NEW:
%     tuningScenario = 'SPEAR-SWGCP';
%     if isunix
%         dist_file = '/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/spear/random-SPEAR-SWGCP-JAIR-results-1000train-1000test.txt';
%         random_config_file = '/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/spear/1000samples-algospear1.2.1.1-runobjruntime-overallobjmean10-runs1000-time300.0-length2147483647_0.txt'
%     else
%         dist_file = 'Z:\arrowspace\ParamILS\spear\random-SPEAR-SWGCP-JAIR-results-1000train-1000test.txt';
%         random_config_file = 'Z:\arrowspace\ParamILS\spear\1000samples-algospear1.2.1.1-runobjruntime-overallobjmean10-runs1000-time5.0-length2147483647_0.txt';
%     end
%     %analyze_scenario(tuningScenario, dist_file, 5);
%     model_perf_scenario(dist_file, random_config_file, tuningScenario, 5);
%     %piac_potential(dist_file, random_config_file, tuningScenario); %1000
%     clear all;
%     close all hidden;
%
%
%     tuningScenario = 'SPEAR-QCP';
%     if isunix
%         dist_file = '/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/spear/random-QCP-JAIR-results-1000train-1000test.txt';
%         random_config_file = '/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/spear/1000samples-algospear1.2.1.1-runobjruntime-overallobjmean10-runs1000-time5.0-length2147483647_0.txt';
%     else
%         dist_file = 'Z:\arrowspace\ParamILS\spear\random-QCP-JAIR-results-1000train-1000test.txt';
%         random_config_file = 'Z:\arrowspace\ParamILS\spear\1000samples-algospear1.2.1.1-runobjruntime-overallobjmean10-runs1000-time5.0-length2147483647_0.txt';
%     end
%     %analyze_scenario(tuningScenario, dist_file, 5);
%     model_perf_scenario(dist_file, random_config_file, tuningScenario, 5);
%     clear all;
%     close all hidden;
%
%
%
%
%     % NEW:
%     tuningScenario = 'SPEAR-swv-al';
%     if isunix
%         dist_file = '/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/spear/random-SPEAR-SWV-results-100inst.txt';
%         random_config_file = '/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/spear/1000samples-algospear1.2.1.1-runobjruntime-overallobjmean10-runs1000-time300.0-length2147483647_0.txt'
%     else
%         dist_file = 'Z:\arrowspace\ParamILS\spear\random-SPEAR-SWV-results-100inst.txt';
%         random_config_file = 'Z:\arrowspace\ParamILS\spear\1000samples-algospear1.2.1.1-runobjruntime-overallobjmean10-runs1000-time300.0-length2147483647_0.txt';
%     end
%     model_perf_scenario(dist_file, random_config_file, tuningScenario, 300);
%     %model_perf_scenario_single_matrix(dist_file, random_config_file, tuningScenario, 300);
%     clear all;
%     close all hidden;
%
%
%     % NEW:
%     tuningScenario = 'CPLEX-regions100';
%     if isunix
%         dist_file = '/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/cplex/random-CPLEX-CATS100-JAIR-results-1000train-1000test.txt';
%         %random_config_file = '/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/cplex/1000-random-configs-early-random-runs1-runobjruntime-overallobjmean-time5.0-tunerTime18000.0-algocplex-stats_0.txt';
%         random_config_file = '/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/cplex/1000random-configs-runobjruntime-overallobjmean-time5.0-tunerTime18000.0-algocplex-stats_0.txt';
%     else
%         dist_file = 'Z:\arrowspace\ParamILS\cplex\random-CPLEX-CATS100-JAIR-results-1000train-1000test.txt';
%         %random_config_file = 'Z:\arrowspace\ParamILS\cplex\1000-random-configs-early-random-runs1-runobjruntime-overallobjmean-time5.0-tunerTime18000.0-algocplex-stats_0.txt';
%         random_config_file = 'Z:\arrowspace\ParamILS\cplex\1000random-configs-runobjruntime-overallobjmean-time5.0-tunerTime18000.0-algocplex-stats_0.txt';
%     end
%     %analyze_scenario(tuningScenario, dist_file, 5);
%     model_perf_scenario(dist_file, random_config_file, tuningScenario, 5);
%     %model_perf_scenario_single_matrix(dist_file, random_config_file, tuningScenario, 5);
%     clear all;
%     close all hidden;
%
%     % NEW:
%     tuningScenario = 'SPEAR-ibm-al';
%     if isunix
%         dist_file = '/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/spear/SPEAR-ibm-al-pred_mean_matrix.txt';
%         random_config_file = '/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/spear/1000samples-algospear1.2.1.1-runobjruntime-overallobjmean10-runs1000-time300.0-length2147483647_0.txt'
%     else
%         dist_file = 'Z:\arrowspace\ParamILS\spear\random-SPEAR-IBM-results-100inst.txt';
%         random_config_file = 'Z:\arrowspace\ParamILS\spear\1000samples-algospear1.2.1.1-runobjruntime-overallobjmean10-runs1000-time300.0-length2147483647_0.txt';
%     end
%     %analyze_scenario(tuningScenario, dist_file, 300); %1000
%     model_perf_scenario(dist_file, random_config_file, tuningScenario, 300);
%     %model_perf_scenario_single_matrix(dist_file, random_config_file, tuningScenario, 300);
%
%     clear all;
%     close all hidden;
%
%
%     % NEW:
%     tuningScenario = 'CPLEX-OR-al';
%     if isunix
%         dist_file = '/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/cplex/random-CPLEX-Orlib-results.txt';
%         %random_config_file = '/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/cplex/1000-random-configs-early-random-runs1-runobjruntime-overallobjmean-time5.0-tunerTime18000.0-algocplex-stats_0.txt';
%         random_config_file = '/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/cplex/1000random-configs-runobjruntime-overallobjmean-time300.0-tunerTime18000.0-algocplex-stats_0.txt';
%     else
%         dist_file = 'Z:\arrowspace\ParamILS\cplex\random-CPLEX-Orlib-results.txt';
%         %random_config_file = 'Z:\arrowspace\ParamILS\cplex\1000-random-configs-early-random-runs1-runobjruntime-overallobjmean-time5.0-tunerTime18000.0-algocplex-stats_0.txt';
%         random_config_file = 'Z:\arrowspace\ParamILS\cplex\1000random-configs-runobjruntime-overallobjmean-time300.0-tunerTime18000.0-algocplex-stats_0.txt';
%     end
%     %analyze_scenario(tuningScenario, dist_file, 300); %1000
%     %piac_potential(dist_file, random_config_file, tuningScenario); %1000
%
%     model_perf_scenario(dist_file, random_config_file, tuningScenario, 300);
%     %model_perf_scenario_single_matrix(dist_file, random_config_file, tuningScenario, 300);
%
%     clear all;
%     close all hidden;
%
%
%
%
%
%
%
%
%
%
%
%     tuningScenario = 'SAPS-QCP-cont';
%     if isunix
%         dist_file = '/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/saps-newqcp/non-trivial-random-newQCP-JAIR-results-1000train-1000test.txt';
%     %    random_config_file = '/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/saps-newqcp/1000random_samples-runobjruntime-overallobjmean10-time5.0-algosaps-later-clean.txt';
%         random_config_file = '/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/saps-newqcp/1000random_samples-runobjruntime-overallobjmean10-time5.0-algosaps-later-clean.txt';
%     else
%         dist_file = 'Z:\arrowspace\ParamILS\saps-newqcp\non-trivial-random-newQCP-JAIR-results-1000train-1000test.txt';
%     %    random_config_file = 'Z:\arrowspace\ParamILS\saps-newqcp\1000random_samples-runobjruntime-overallobjmean10-time5.0-algosaps-later-clean.txt';
%         random_config_file = 'Z:\arrowspace\ParamILS\saps-newqcp\1000random_samples-runobjruntime-overallobjmean10-time5.0-algosaps-later-clean.txt';
%     end
%     %analyze_scenario(tuningScenario, dist_file, 5);
%     model_perf_scenario(dist_file, random_config_file, tuningScenario, 5);
%     clear all;
%     close all hidden;
%
% end
%
% % % NEW:
% % tuningScenario = 'SPEAR-swv-al';
% % if isunix
% %     dist_file = '/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/spear/random-SPEAR-SWV-results-100inst.txt';
% %     random_config_file = '/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/spear/1000samples-algospear1.2.1.1-runobjruntime-overallobjmean10-runs1000-time300.0-length2147483647_0.txt'
% % else
% %     dist_file = 'Z:\arrowspace\ParamILS\spear\random-SPEAR-SWV-results-100inst.txt';
% %     random_config_file = 'Z:\arrowspace\ParamILS\spear\1000samples-algospear1.2.1.1-runobjruntime-overallobjmean10-runs1000-time300.0-length2147483647_0.txt';
% % end
% % %analyze_scenario(tuningScenario, dist_file, 300); %1000
% % model_perf_scenario(dist_file, random_config_file, tuningScenario, 300);
% % clear all;
% % close all hidden;
% %
% %
% % % NEW:
% % tuningScenario = 'SPEAR-ibm-al';
% % if isunix
% %     dist_file = '/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/spear/random-SPEAR-IBM-results-100inst.txt';
% %     random_config_file = '/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/spear/1000samples-algospear1.2.1.1-runobjruntime-overallobjmean10-runs1000-time300.0-length2147483647_0.txt'
% % else
% %     dist_file = 'Z:\arrowspace\ParamILS\spear\random-SPEAR-IBM-results-100inst.txt';
% %     random_config_file = 'Z:\arrowspace\ParamILS\spear\1000samples-algospear1.2.1.1-runobjruntime-overallobjmean10-runs1000-time300.0-length2147483647_0.txt';
% % end
% % %analyze_scenario(tuningScenario, dist_file, 300); %1000
% % model_perf_scenario(dist_file, random_config_file, tuningScenario, 300);
% %
% % clear all;
% % close all hidden;
% %
% %
% % % NEW:
% % tuningScenario = 'CPLEX-regions100';
% % if isunix
% %     dist_file = '/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/cplex/random-CPLEX-CATS100-JAIR-results-1000train-1000test.txt';
% %     random_config_file = '/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/cplex/1000-random-configs-early-random-runs1-runobjruntime-overallobjmean-time5.0-tunerTime18000.0-algocplex-stats_0.txt';
% % else
% %     dist_file = 'Z:\arrowspace\ParamILS\cplex\random-CPLEX-CATS100-JAIR-results-1000train-1000test.txt';
% %     random_config_file = 'Z:\arrowspace\ParamILS\cplex\1000-random-configs-early-random-runs1-runobjruntime-overallobjmean-time5.0-tunerTime18000.0-algocplex-stats_0.txt';
% % end
% % %analyze_scenario(tuningScenario, dist_file, 5);
% % model_perf_scenario(dist_file, random_config_file, tuningScenario, 5);
% % clear all;
% % close all hidden;
% %
% %
% %
% %
% %
% %
% %
% % % NEW:
% % tuningScenario = 'SatComp07-phase2-industrial';
% % if isunix
% %     dist_file = '/.autofs/homes/ubccshome/h/hutter/satcomp_results/SAT07-matrix-industrial-phase2.csv';
% % else
% %     dist_file = 'Z:\satcomp_results\SAT07-matrix-industrial-phase2.csv';
% % end
% % analyze_scenario(tuningScenario, dist_file, 5000); %1000
% % clear all;
% % close all hidden;
%
%
% %
% %
% % % NEW:
% % tuningScenario = 'Satenstein-SWGCP';
% % if isunix
% %     %dist_file = '/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/satenstein/SWGCP-500dls389nondls111firstnondls-parameters-runobjruntime-overallobjmean10-time5-algosatenstein-stats-firstinstance_list_0-1000train-1000test.txt';
% %     dist_file = '/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/satenstein/SWGCP-500-random-dls-500-random-nondls-parameters-runobjruntime-overallobjmean10-time5-algosatenstein-stats-firstinstance_list_0-1000train-1000test.txt';
% %     random_config_file = '/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/satenstein/500-random-dls_500-random-nondls-parameters-runobjruntime-overallobjmean10-time5-algosatenstein-stats-cannot-be-evaluated-together-since-different-params-files.txt';
% % else
% %     %dist_file = 'Z:\arrowspace\ParamILS\satenstein\SWGCP-500dls389nondls111firstnondls-parameters-runobjruntime-overallobjmean10-time5-algosatenstein-stats-firstinstance_list_0-1000train-1000test.txt';
% %     dist_file = 'Z:\arrowspace\ParamILS\satenstein\SWGCP-500-random-dls-500-random-nondls-parameters-runobjruntime-overallobjmean10-time5-algosatenstein-stats-firstinstance_list_0-1000train-1000test.txt';
% %     random_config_file = 'Z:\arrowspace\ParamILS\satenstein\500-random-dls_500-random-nondls-parameters-runobjruntime-overallobjmean10-time5-algosatenstein-stats-cannot-be-evaluated-together-since-different-params-files.txt';
% % end
% % analyze_scenario(tuningScenario, dist_file, 5); %1000
% % %piac_potential(dist_file, random_config_file, tuningScenario); %1000
% % clear all;
% % close all hidden;
% %
% % % NEW:
% % tuningScenario = 'Satenstein-QCP';
% % if isunix
% %     %dist_file = '/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/satenstein/QCP-395dls_rep129nondls_repfirst-parameters-runobjruntime-overallobjmean10-time5-algosatenstein-stats-firstinstance_list_0-1000train-1000test.txt';
% %     dist_file = '/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/satenstein/QCP-500-random-dls-500-random-nondls-parameters-runobjruntime-overallobjmean10-time5-algosatenstein-stats-firstinstance_list_0-1000train-1000test.txt';
% %     random_config_file = '/.autofs/csother/ubccsprojectarrow/hutter/ParamILS/satenstein/500-random-dls_500-random-nondls-parameters-runobjruntime-overallobjmean10-time5-algosatenstein-stats-cannot-be-evaluated-together-since-different-params-files.txt';
% % else
% %     %dist_file = 'Z:\arrowspace\ParamILS\satenstein\QCP-395dls_rep129nondls_repfirst-parameters-runobjruntime-overallobjmean10-time5-algosatenstein-stats-firstinstance_list_0-1000train-1000test.txt';
% %     dist_file = 'Z:\arrowspace\ParamILS\satenstein\QCP-500-random-dls-500-random-nondls-parameters-runobjruntime-overallobjmean10-time5-algosatenstein-stats-firstinstance_list_0-1000train-1000test.txt';
% %     random_config_file = 'Z:\arrowspace\ParamILS\satenstein\500-random-dls_500-random-nondls-parameters-runobjruntime-overallobjmean10-time5-algosatenstein-stats-cannot-be-evaluated-together-since-different-params-files.txt';
% % end
% % analyze_scenario(tuningScenario, dist_file, 5); %1000
% % %piac_potential(dist_file, random_config_file, tuningScenario); %1000
% % clear all;
% % close all hidden;
% %
% %
% %
% %
% %
% %
% % % % figure
% % % % dist_file = 'Z:\arrowspace\ParamILS\cplex\random-CATS200-test100-dist-ps3-stats-algocplex-runobjruntime-overallobjmean-runs10-time5.0-init1_0.txt';
% % % % traintest_file = 'Z:\arrowspace\ParamILS\cplex\real\real_random-CATS200-ps3-stats-algocplex-runobjruntime-overallobjmean-runs10-time5.0-init1_0-instance_list_0.txt.txt';
% % % % analyze_scenario('CPLEX on regions 200', traintest_file, dist_file)
% % %
% % % figure
% % % dist_file = 'Z:\arrowspace\ParamILS\saps-qcp\random-QCP-test100-dist-ps3-stats-algosaps-runobjruntime-overallobjmean-runs10-time5.0-init1_0.txt';
% % % traintest_file = 'Z:\arrowspace\ParamILS\saps-qcp\real\real_random-ps3-stats-algosaps-runobjruntime-overallobjmean-runs10-time5.0-init1_0-instance_list_0.txt.txt';
% % % analyze_scenario('SAPS on QCP', traintest_file, dist_file, 5)
% % %
% % % figure
% % % dist_file = 'Z:\arrowspace\ParamILS\spear\random-SWGCP-test100-dist-ps3-stats-algospear-runobjruntime-overallobjmean-runs10-time5.0-init1_0.txt';
% % % traintest_file = 'Z:\arrowspace\ParamILS\spear\real\real_random-SWGCP-ps3-stats-algospear1.2.1.1-runobjruntime-overallobjmean-runs10-time5.0-init1_0-instance_list_0.txt.txt';
% % % analyze_scenario('SPEAR on SWGCP', traintest_file, dist_file, 5)
% % %
% % % figure
% % % dist_file = 'Z:\arrowspace\ParamILS\saps-sw\random-SWGCP-test100-dist-ps3-stats-algosaps-runobjruntime-overallobjmean-runs10-time5.0-init1_0.txt';
% % % traintest_file = 'Z:\arrowspace\ParamILS\saps-sw\real\real_random-ps3-stats-algosaps-runobjruntime-overallobjmean-runs10-time5.0-init1_0-instance_list_0.txt.txt';
% % % analyze_scenario('SAPS on SWGCP', traintest_file, dist_file, 5)
% % %
% % %
% % % figure
% % % dist_file = 'Z:\arrowspace\ParamILS\spear\random-QCP-test100-dist-ps3-stats-algospear-runobjruntime-overallobjmean-runs10-time5.0-init1_0.txt';
% % % traintest_file = 'Z:\arrowspace\ParamILS\spear\real\real_random-QCP-ps3-stats-algospear1.2.1.1-runobjruntime-overallobjmean-runs10-time5.0-init1_0-instance_list_0.txt.txt';
% % % analyze_scenario('SPEAR on QCP', traintest_file, dist_file, 5)
% % %
% % %
% % % % %N=100
% % % % figure
% % % % traintest_file = 'Z:\arrowspace\ParamILS\cplex\real\real_random-ps3-stats-algocplex-runobjruntime-overallobjmean-runs100-time5.0-init1_0-instance_list_0.txt.txt';
% % % % analyze_scenario('CPLEX on regions 100, N=100', traintest_file)
% % % %
% % % % figure
% % % % analyze_scenario('SPEAR on SWGCP, N=100', 'Z:\arrowspace\ParamILS\spear\real\real_random-ps3-stats-algospear1.2.1.1-runobjruntime-overallobjmean-runs100-time5.0-init1_0-instance_list_0.txt.txt')
% % % %
% % % % figure
% % % % analyze_scenario('SAPS on SWGCP, N=100',
% % % % 'Z:\arrowspace\ParamILS\saps-sw\real\real_random-ps3-stats-algosaps-runobjruntime-overallobjmean-runs100-time5.0-init1_0-instance_list_0.txt.txt')


%=== DECLARATION OF ALL ANONYMOUS FUNCTIONS FOR MCC.
%#function covSum
%#function covDIFFard
%#function covDIFFiso
%#function covHybridSE_DIFFard
%#function covHybridSE_DIFFiso
%#function covHybridSE_DIFFiso_OnlyLen
%#function covHybridSE_DIFFiso_NoLen
%#function covHybridSE_DIFFiso_algo
%#function covMatern3iso
%#function covMatern5iso
%#function covNoise
%#function covSEard
%#function covSEiso
%#function covSum
%#function gpr
%#function gprCensor
%#function negLogCensoredConditional
%#function branin.m
%#function disc_bnet_ll
%#function disc_test_fun2
%#function disc_test_fun
%#function griewank
%#function hart6
%#function hartman6
%#function hartman6_marginalized
%#function hartman6_marginalized_t
%#function mixed_disc_cont_testfun
%#function prod_of_branins
%#function saved_saps_swgcp_first100train
%#function simplest_discrete
%#function sixHumpCamelBack
%#function neg_log_exp_imp_model
%#function eval_algo
%#function eval
%#function nmllModel
%#function crossValLL
%#function mu_plus_sigma_model_one_inst
%#function neg_sample_exp_imp_model
%#function sample_exp_imp_forest
%#function gaussian_exp_imp_forest_m
%#function avg_minus_sigma_pred_model
%#function mu_minus_k_sigma_pred_model
%#function quantile_pred_model
%#function cmaes_fsphere
%#function cmaes_rastrigin
%#function cmaes_griewank
%#function cmaes_ackley
%#function cmaes323_sphere_10
%#function cmaes323_ackley_10
%#function cmaes323_griewank_10
%#function cmaes323_rastrigin_10
%#function cmaes323_bf1_10
%#function cmaes323_bf2_10
%#function cmaes323_bf3_10
%#function cmaes323_bf4_10
%#function cmaes323_bf5_10
%#function cmaes323_bf6_10
%#function cmaes323_bf7_10
%#function cmaes323_bf8_10
%#function cmaes323_bf9_10
%#function cmaes323_bf10_10
%#function cmaes323_bf11_10
%#function cmaes323_bf12_10
%#function benchmark_func
%#function benchmark_func_1
%#function benchmark_func_2
%#function benchmark_func_3
%#function benchmark_func_4
%#function benchmark_func_5
%#function benchmark_func_6
%#function benchmark_func_7
%#function benchmark_func_8
%#function benchmark_func_9
%#function benchmark_func_10
%#function benchmark_func_11
%#function benchmark_func_12
%#function regpoly0
%#function corrgauss
%#function cmaes
%#function mlppak
%#function mlpunpak
%#function scg
%#function neterr
%#function mlperr
%#function netgrad
%#function mlpgrad
