function output_vimps(savefile, options_vec, prefix, xstring)
if nargin < 4
    xstring = 'Feature subset size';
end

numSamples = 10;
for model_idx = 1:length(options_vec)
    for numSample=1:numSamples
        this_savefile = strcat([savefile, '_', num2str(numSample)]);
        savefilename = strcat([this_savefile, '_', options_vec{model_idx}.unique_model_name]);
        try
            load(savefilename, 'vimp');
            all_vimps{model_idx,numSample} = vimp; %#ok<AGROW>
        catch ME
            all_vimps{model_idx,numSample} = -inf; %#ok<AGROW>
        end
    end
end

ready = 0;
for model_idx = 1:length(options_vec)
    for numSample = 1:numSamples
        if isfield(all_vimps{model_idx,numSample}, 'featureNames')
            featureNames = all_vimps{1,1}.featureNames;
            numFeatures = length(featureNames);
            ready = 1;
        end
    end
end
if ~ready
    return
end

for model_idx = 1:length(options_vec)
    count = 0;
    importance = [];
    for numSample = 1 % only take 1 sample in order to ensure showing a complete sufficient set
        if isfield(all_vimps{model_idx,numSample}, 'featureNames')
            count = count + 1;
            importance(all_vimps{model_idx,numSample}.selected_feature_idxs, count) = all_vimps{model_idx,numSample}.importance;
            importance(find(importance(:,count)>99.99),count) = 100;
            importance(find(importance(:,count)<0.01),count) = 0;
        end
    end
    mean_importance = mean(importance,2);
    numSample = 1;
    vimp = all_vimps{model_idx,numSample};
    for i=1:length(vimp.selected_feature_indices_in_order_added)
        fprintf(strcat([vimp.featureNames{vimp.selected_feature_indices_in_order_added(i)}, ' (%.1f) ']), round_to_digits(vimp.importance(i),1) );
        if i < length(vimp.selected_feature_indices_in_order_added)
            fprintf(' & ');
        else
            fprintf('\n');
        end
    end
end
numFeaturesSelected = length(vimp.selected_feature_indices_in_order_added);

if 0
    return
end


%% Code for generating boxplots over multiple samples. Kevin does not like
%% that visualization since it does not show sufficient sets - in fact,
%% sometimes it shows 2 or 3 variables ranging from 0 to 100 in importance
%% that are almost certainly very correlated.
%for model_idx = 1:length(options_vec)
%     count = 0;
%     for numSample = 1:numSamples
%         if isfield(all_vimps{model_idx,numSample}, 'featureNames')
%             count = count + 1;
%             importance(all_vimps{model_idx,numSample}.selected_feature_idxs, count) = all_vimps{model_idx,numSample}.importance;
%             importance(find(importance(:,count)>99.99),count) = 100;
%             importance(find(importance(:,count)<0.01),count) = 0;
%         end
%     end
%     mean_importance = mean(importance,2);
%     [tmp, sort_idx] = sort(-mean_importance);
%     k = 10;
%     idx = sort_idx(k:-1:1);
%     figure
%     % make sure we can already plot when we only have one datapoint each.
%     if size(importance,2)==1
%         importance = [importance,importance];
%     end
%     myboxplot( importance(idx(1:k),:)', 'orientation', 'horizontal' );
% %     for i=1:k
% %         plot(importance(idx(i),:), ones(numSamples,1)*i, 'ko');
% %         hold on
% %     end
% 	set(gca, 'YTick', 1:k);
% 	set(gca,'YTickLabel',featureNames(idx(1:k)), 'FontSize', 14);
%     ylabel('');
%     xlabel('Importance');
%     axis([-5 105 0.001 k+1-0.001])
%     
%     figure_prefix = strcat([prefix, '-', options_vec{model_idx}.unique_model_name, '-varImp']);
%     save_gca_as_fig_and_pdf(figure_prefix);
% end

%% Plot RMSE as a function of subset size over the course of fwd. sel.
figure
hXLabel=xlabel(xstring);
hold on;
hYLabel=ylabel('RMSE');

plot_full_rmse = 1;

maxrmse = -inf;
minrmse = inf;
hE = [];
hE2 = [];
legendstr = {};
for model_idx = 1:length(options_vec)
    mean_rmses = 0;
    mean_full_rmse = 0;
    mean_emptyModelRMSE = 0;
    count = 0;
    for numSample = 1:numSamples
        vimp = all_vimps{model_idx,numSample};
        if isfield(vimp, 'featureNames')
            mean_rmses = mean_rmses + vimp.rmses;
            mean_full_rmse = mean_full_rmse + vimp.full_rmse;
            mean_emptyModelRMSE = mean_emptyModelRMSE + vimp.emptyModelRMSE;
            count = count + 1;
        end
    end
    mean_emptyModelRMSE = mean_emptyModelRMSE/count;
    mean_rmses = mean_rmses/count;
    
    numFeaturesSelected = 5;
    mean_rmses = mean_rmses(1:numFeaturesSelected); % only plot the first 5
    
    all_rmses = [mean_emptyModelRMSE, mean_rmses];
    full_rmse = mean_full_rmse/count;
    hE(end+1)=plot(0:length(mean_rmses), all_rmses);
    hold on;
    maxrmse = max(max(maxrmse, max(all_rmses)),full_rmse);
    minrmse = min(min(minrmse, min(all_rmses)),full_rmse);
    if plot_full_rmse
        hE2(end+1)=line([-100,100],[full_rmse,full_rmse]);
        maxrmse = max(maxrmse, full_rmse);
        minrmse = min(minrmse, full_rmse);
    end
    legendstr{end+1} = options_vec{model_idx}.unique_model_name;
end

% for i=1:length(vimp.rmses)
%     rmses = vimp.rmses{i};
%     hE(i)=plot(1:length(rmses), rmses);
%     hold on;
%     hE2(i)=line([0,100],[vimp.full_rmse(i),vimp.full_rmse(i)]);
% end

set(hE(1)                      , ...
  'LineStyle'       , '--'      , ...
  'LineWidth'       , 2           , ...
  'Color'           , [.4 .4 .4]  , ...
  'MarkerSize'      , 6           , ...
  'Marker'          , 'o'         , ...
  'MarkerFaceColor' , [.7 .7 .7]  , ...
  'MarkerEdgeColor' , [.4 .4 .4]);
if plot_full_rmse
    set(hE2(1), 'LineStyle', '-', ...
  'LineStyle'       , '--'      , ...
  'LineWidth'       , 1           , ...
  'Color'           , [.4 .4 .4]  , ...
  'MarkerSize'      , 6           , ...
  'Marker'          , 'o'         , ...
  'MarkerFaceColor' , [.7 .7 .7]  , ...
  'MarkerEdgeColor' , [.4 .4 .4]);
end


if length(hE) > 1
    set(hE(2)                            , ...
        'Color'           , 'b'  , ...
        'Marker'          , 'x'         , ...
        'LineStyle'       , '-'      , ...
        'MarkerSize'      , 12, ...        
        'LineWidth'       , 2);
    if plot_full_rmse
        set(hE2(2), 'LineStyle', '-', ...
        'Color'           , 'b'  , ...
            'Marker'          , 'x'         , ...
            'MarkerSize'      , 12, ...        
            'LineWidth'       , 1);
    end

    if length(hE) > 2
        set(hE(3)                            , ...
            'Color'           , 'r'  , ...
            'Marker'          , '.'         , ...
            'LineStyle'       , ':'      , ...
            'MarkerSize'      , 16, ...        
            'LineWidth'       , 2);
        if plot_full_rmse
             set(hE2(3), 'LineStyle', ':',  ...
            'Color'           , 'r'  , ...
            'Marker'          , '.'         , ...
            'MarkerSize'      , 16, ...        
            'LineWidth'       , 1);
        end
        
        if length(hE) > 3
            set([hE(4),hE2(4)]                            , ...
                'Color'           , 'k'  , ...
                'Marker'          , 'd'         , ...
                'LineStyle'       , '-.'      , ...
                'MarkerSize'      , 9, ...        
                'LineWidth'       , 2);
            set(hE2(4), 'LineStyle', '-');

            if length(hE) > 4
                set([hE(5),hE2(5)]                            , ...
                    'Color'           , 'g'  , ...
                    'Marker'          , 's'         , ...
                    'LineStyle'       , ':'      , ...
                    'MarkerSize'      , 10, ...        
                    'LineWidth'       , 2);
                set(hE2(5), 'LineStyle', '-');
                
            end
        end
    end
end

len = length(options_vec);
legendstr = legendstr(len:-1:1);
hE = hE(len:-1:1);

slack = (maxrmse-minrmse) / 10;
%axis([0.001 11-0.001 max(0,minrmse-0.05) (maxrmse+0.05)*1.05])
%axis([0.001 11-0.001 max(0,(minrmse-slack)/1) (maxrmse+slack)*1])
axis([0.001-1 numFeaturesSelected+1-0.001 0 (maxrmse+slack)])

legendPos = 'SouthWest';
% legend(hE, legendstr, 'location', legendPos);

fontSize = 24;
set([hXLabel, hYLabel], 'FontSize', fontSize);
set(gca, ...
  'Box'         , 'off'     , ...
  'TickDir'     , 'out'     , ...
  'XMinorTick'  , 'on'      , ...
  'YMinorTick'  , 'on'      , ...
  'XColor'      , [0 0 0], ...
  'YColor'      , [0 0 0], ...
  'FontSize'    , fontSize-10, ...
  'LineWidth'   , 2         );
set(gcf, 'Outerposition', [0,0,500,500]);
%   'XTick'       , 10.^[-10:10], ...
%   'YTick'       , 10.^[-10:10], ...

figure_prefix = strcat([prefix, '-', options_vec{model_idx}.unique_model_name, '-rmses_with_subsets']);
save_gca_as_fig_and_pdf(figure_prefix);
